/*
 *      p8-2.c
 *      Beispielprogramm 2, Abschnitt 8
 *      Haeufigkeit von Woertern zaehlen
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define MAXWORD 32
#define LETTER  'a'
#define DIGIT   '0'

/* binaerer Baum */
struct tnode {
  char *word;           /* zeigt auf den Text */
  int count;            /* Haeufigkeit */
  struct tnode *left;   /* linker Nachfolger */
  struct tnode *right;  /* rechter Nachfolger */
};
      
struct tnode *tree(struct tnode *, char *);
void treeprint(struct tnode *p);
struct tnode *talloc(void);
char *strsave(char *);
int getword(char *, int);
int type(int);

void main(void)
{
  struct tnode *root;
  char word[MAXWORD];
  int t;
  
  root = NULL;
  while ((t = getword(word, MAXWORD)) != EOF)
    if (t == LETTER)
      root = tree(root, word);
      
  treeprint(root);

} /* main() */

/* w bei oder nach p einfuegen */
struct tnode *tree(struct tnode *p, char *w)
{
  int cond;
  
  if (p == NULL) {      /* ein neues Wort */
    p = talloc();       /* einen neuen Knoten anlegen */
    p->word = strsave(w);
    p->count = 1;
    p->left = p->right = NULL;
  } 
  else if ((cond = strcmp(w, p->word)) == 0)
    p->count++;         /* Wort wird wiederholt */
  else if (cond < 0)    /* kleiner, links darunter */
    p->left = tree(p->left, w);
  else                  /* groesser, rechts darunter */
    p->right = tree(p->right, w);
  return(p);
  
} /* tree() */

/* Baum p rekursiv ausgeben */
void treeprint(struct tnode *p)
{
  if (p != NULL) {
    treeprint(p->left);
    printf("%4d %s\n",p->count, p->word);
    treeprint(p->right);
  }
  
} /* treeprint() */

/* Platz fr eine Struktur besorgen */
struct tnode *talloc(void)
{
  return((struct tnode *) calloc(1, sizeof(struct tnode)));
  
} /* talloc() */

/* Zeichenkette s in Sicherheit bringen */
char *strsave(char *s)
{
  char *p;
  
  if ((p = (char *) malloc(strlen(s) + 1)) != NULL)
    strcpy(p, s);
  return (p);
  
} /* strsave() */

#define getch() getc(stdin)
#define ungetch(c) ungetc(c, stdin)
 
/* naechstes Wort aus der Eingabe holen */
int getword(char *w, int lim)
{
  int c, t;
  
  if (type(c = *w++ = getch()) != LETTER) {
    *w = '\0';
    return(c);
  }
  while (--lim > 0) {
    t = type(c= *w++ = getch());
    if (t != LETTER && t != DIGIT) {
      ungetch(c);
      break;
    }
  }
  *(w-1) = '\0';
  return(LETTER);

} /* getword() */

/* Typ eines ASCII Zeichens liefern */
int type(int c)
{
  if (c >= 'a' && c <= 'z' || c >= 'A' && c <= 'Z')
    return(LETTER);
  else if (c >= '0' && c <= '9')
    return(DIGIT);
  else
    return(c);

} /* type() */
